# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing
import urllib.parse
from json.decoder import JSONDecodeError

import httpx
import pydantic

from .....core.api_error import ApiError
from .....core.datetime_utils import serialize_datetime
from .....core.jsonable_encoder import jsonable_encoder
from .....core.remove_none_from_headers import remove_none_from_headers
from .....environment import CandidApiEnvironment
from ....billing_notes.types.billing_note_base import BillingNoteBase
from ....claims.types.claim_status import ClaimStatus
from ....commons.types.date import Date
from ....commons.types.encounter_external_id import EncounterExternalId
from ....commons.types.encounter_id import EncounterId
from ....commons.types.facility_type_code import FacilityTypeCode
from ....commons.types.page_token import PageToken
from ....commons.types.street_address_long_zip import StreetAddressLongZip
from ....commons.types.work_queue_id import WorkQueueId
from ....diagnoses.types.diagnosis_create import DiagnosisCreate
from ....diagnoses.types.diagnosis_id import DiagnosisId
from ....encounter_providers.resources.v_2.types.billing_provider import BillingProvider
from ....encounter_providers.resources.v_2.types.referring_provider import ReferringProvider
from ....encounter_providers.resources.v_2.types.rendering_provider import RenderingProvider
from ....individual.types.patient_create import PatientCreate
from ....individual.types.subscriber_create import SubscriberCreate
from ....service_facility.types.encounter_service_facility_base import EncounterServiceFacilityBase
from ....service_lines.types.service_line_create import ServiceLineCreate
from ....tags.types.tag_id import TagId
from .types.clinical_note_category_create import ClinicalNoteCategoryCreate
from .types.encounter import Encounter
from .types.encounter_attachment import EncounterAttachment
from .types.encounter_page import EncounterPage
from .types.encounter_sort_options import EncounterSortOptions
from .types.generate_clinical_notes_pdf_response import GenerateClinicalNotesPdfResponse
from .types.intervention import Intervention
from .types.mark_as_not_billable_response import MarkAsNotBillableResponse
from .types.medication import Medication
from .types.network_status_computation_results import NetworkStatusComputationResults
from .types.patient_history_category import PatientHistoryCategory
from .types.prior_authorization_number import PriorAuthorizationNumber
from .types.synchronicity_type import SynchronicityType
from .types.vitals import Vitals

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class V4Client:
    def __init__(
        self, *, environment: CandidApiEnvironment = CandidApiEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        claim_status: typing.Optional[ClaimStatus] = None,
        sort: typing.Optional[EncounterSortOptions] = None,
        page_token: typing.Optional[PageToken] = None,
        date_of_service_min: typing.Optional[Date] = None,
        date_of_service_max: typing.Optional[Date] = None,
        primary_payer_names: typing.Optional[str] = None,
        search_term: typing.Optional[str] = None,
        external_id: typing.Optional[EncounterExternalId] = None,
        diagnoses_updated_since: typing.Optional[dt.datetime] = None,
        tag_ids: typing.Union[typing.Optional[TagId], typing.List[TagId]],
        do_not_bill: typing.Optional[bool] = None,
        work_queue_id: typing.Optional[WorkQueueId] = None,
    ) -> EncounterPage:
        _response = httpx.request(
            "GET",
            urllib.parse.urljoin(f"{self._environment.value}/", "api/encounters/v4"),
            params={
                "limit": limit,
                "claim_status": claim_status,
                "sort": sort,
                "page_token": page_token,
                "date_of_service_min": date_of_service_min,
                "date_of_service_max": date_of_service_max,
                "primary_payer_names": primary_payer_names,
                "search_term": search_term,
                "external_id": external_id,
                "diagnoses_updated_since": serialize_datetime(diagnoses_updated_since)
                if diagnoses_updated_since is not None
                else None,
                "tag_ids": tag_ids,
                "do_not_bill": do_not_bill,
                "work_queue_id": work_queue_id,
            },
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EncounterPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get(self, encounter_id: EncounterId) -> Encounter:
        _response = httpx.request(
            "GET",
            urllib.parse.urljoin(f"{self._environment.value}/", f"api/encounters/v4/{encounter_id}"),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Encounter, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def create(
        self,
        *,
        patient: PatientCreate,
        billing_provider: BillingProvider,
        rendering_provider: RenderingProvider,
        referring_provider: typing.Optional[ReferringProvider] = OMIT,
        service_facility: typing.Optional[EncounterServiceFacilityBase] = OMIT,
        subscriber_primary: typing.Optional[SubscriberCreate] = OMIT,
        subscriber_secondary: typing.Optional[SubscriberCreate] = OMIT,
        diagnoses: typing.List[DiagnosisCreate],
        clinical_notes: typing.Optional[typing.List[ClinicalNoteCategoryCreate]] = OMIT,
        billing_notes: typing.Optional[typing.List[BillingNoteBase]] = OMIT,
        place_of_service_code: FacilityTypeCode,
        patient_histories: typing.Optional[typing.List[PatientHistoryCategory]] = OMIT,
        service_lines: typing.Optional[typing.List[ServiceLineCreate]] = OMIT,
        external_id: typing.Optional[EncounterExternalId] = OMIT,
        date_of_service: Date,
        end_date_of_service: typing.Optional[Date] = OMIT,
        prior_authorization_number: typing.Optional[PriorAuthorizationNumber] = OMIT,
        patient_authorized_release: bool,
        benefits_assigned_to_provider: bool,
        provider_accepts_assignment: bool,
        appointment_type: typing.Optional[str] = OMIT,
        do_not_bill: typing.Optional[bool] = OMIT,
        existing_medications: typing.Optional[typing.List[Medication]] = OMIT,
        vitals: typing.Optional[Vitals] = OMIT,
        interventions: typing.Optional[typing.List[Intervention]] = OMIT,
        pay_to_address: typing.Optional[StreetAddressLongZip] = OMIT,
        synchronicity: typing.Optional[SynchronicityType] = OMIT,
    ) -> Encounter:
        _request: typing.Dict[str, typing.Any] = {
            "patient": patient,
            "billing_provider": billing_provider,
            "rendering_provider": rendering_provider,
            "diagnoses": diagnoses,
            "place_of_service_code": place_of_service_code,
            "date_of_service": date_of_service,
            "patient_authorized_release": patient_authorized_release,
            "benefits_assigned_to_provider": benefits_assigned_to_provider,
            "provider_accepts_assignment": provider_accepts_assignment,
        }
        if referring_provider is not OMIT:
            _request["referring_provider"] = referring_provider
        if service_facility is not OMIT:
            _request["service_facility"] = service_facility
        if subscriber_primary is not OMIT:
            _request["subscriber_primary"] = subscriber_primary
        if subscriber_secondary is not OMIT:
            _request["subscriber_secondary"] = subscriber_secondary
        if clinical_notes is not OMIT:
            _request["clinical_notes"] = clinical_notes
        if billing_notes is not OMIT:
            _request["billing_notes"] = billing_notes
        if patient_histories is not OMIT:
            _request["patient_histories"] = patient_histories
        if service_lines is not OMIT:
            _request["service_lines"] = service_lines
        if external_id is not OMIT:
            _request["external_id"] = external_id
        if end_date_of_service is not OMIT:
            _request["end_date_of_service"] = end_date_of_service
        if prior_authorization_number is not OMIT:
            _request["prior_authorization_number"] = prior_authorization_number
        if appointment_type is not OMIT:
            _request["appointment_type"] = appointment_type
        if do_not_bill is not OMIT:
            _request["do_not_bill"] = do_not_bill
        if existing_medications is not OMIT:
            _request["existing_medications"] = existing_medications
        if vitals is not OMIT:
            _request["vitals"] = vitals
        if interventions is not OMIT:
            _request["interventions"] = interventions
        if pay_to_address is not OMIT:
            _request["pay_to_address"] = pay_to_address
        if synchronicity is not OMIT:
            _request["synchronicity"] = synchronicity
        _response = httpx.request(
            "POST",
            urllib.parse.urljoin(f"{self._environment.value}/", "api/encounters/v4"),
            json=jsonable_encoder(_request),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Encounter, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def update(
        self,
        encounter_id: EncounterId,
        *,
        prior_authorization_number: typing.Optional[PriorAuthorizationNumber] = OMIT,
        external_id: typing.Optional[EncounterExternalId] = OMIT,
        date_of_service: typing.Optional[Date] = OMIT,
        diagnosis_ids: typing.Optional[typing.List[DiagnosisId]] = OMIT,
        tag_ids: typing.Optional[typing.List[TagId]] = OMIT,
        clinical_notes: typing.Optional[typing.List[ClinicalNoteCategoryCreate]] = OMIT,
        pay_to_address: typing.Optional[StreetAddressLongZip] = OMIT,
    ) -> Encounter:
        _request: typing.Dict[str, typing.Any] = {}
        if prior_authorization_number is not OMIT:
            _request["prior_authorization_number"] = prior_authorization_number
        if external_id is not OMIT:
            _request["external_id"] = external_id
        if date_of_service is not OMIT:
            _request["date_of_service"] = date_of_service
        if diagnosis_ids is not OMIT:
            _request["diagnosis_ids"] = diagnosis_ids
        if tag_ids is not OMIT:
            _request["tag_ids"] = tag_ids
        if clinical_notes is not OMIT:
            _request["clinical_notes"] = clinical_notes
        if pay_to_address is not OMIT:
            _request["pay_to_address"] = pay_to_address
        _response = httpx.request(
            "PATCH",
            urllib.parse.urljoin(f"{self._environment.value}/", f"api/encounters/v4/{encounter_id}"),
            json=jsonable_encoder(_request),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Encounter, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def get_attachments(self, encounter_id: EncounterId) -> typing.List[EncounterAttachment]:
        _response = httpx.request(
            "GET",
            urllib.parse.urljoin(f"{self._environment.value}/", f"api/encounters/v4/{encounter_id}/attachments"),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[EncounterAttachment], _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def generate_clinical_notes_pdf(self, encounter_id: EncounterId) -> GenerateClinicalNotesPdfResponse:
        _response = httpx.request(
            "POST",
            urllib.parse.urljoin(
                f"{self._environment.value}/", f"api/encounters/v4/{encounter_id}/clinical-notes-pdf/generate"
            ),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GenerateClinicalNotesPdfResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def recompute_network_status_results(self, encounter_id: EncounterId) -> NetworkStatusComputationResults:
        _response = httpx.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._environment.value}/", f"api/encounters/v4/{encounter_id}/network-status-results"
            ),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(NetworkStatusComputationResults, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    def mark_as_not_billable(self, encounter_id: EncounterId) -> MarkAsNotBillableResponse:
        _response = httpx.request(
            "PUT",
            urllib.parse.urljoin(
                f"{self._environment.value}/", f"api/encounters/v4/{encounter_id}/mark-as-not-billable"
            ),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(MarkAsNotBillableResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV4Client:
    def __init__(
        self, *, environment: CandidApiEnvironment = CandidApiEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    async def get_all(
        self,
        *,
        limit: typing.Optional[int] = None,
        claim_status: typing.Optional[ClaimStatus] = None,
        sort: typing.Optional[EncounterSortOptions] = None,
        page_token: typing.Optional[PageToken] = None,
        date_of_service_min: typing.Optional[Date] = None,
        date_of_service_max: typing.Optional[Date] = None,
        primary_payer_names: typing.Optional[str] = None,
        search_term: typing.Optional[str] = None,
        external_id: typing.Optional[EncounterExternalId] = None,
        diagnoses_updated_since: typing.Optional[dt.datetime] = None,
        tag_ids: typing.Union[typing.Optional[TagId], typing.List[TagId]],
        do_not_bill: typing.Optional[bool] = None,
        work_queue_id: typing.Optional[WorkQueueId] = None,
    ) -> EncounterPage:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "GET",
                urllib.parse.urljoin(f"{self._environment.value}/", "api/encounters/v4"),
                params={
                    "limit": limit,
                    "claim_status": claim_status,
                    "sort": sort,
                    "page_token": page_token,
                    "date_of_service_min": date_of_service_min,
                    "date_of_service_max": date_of_service_max,
                    "primary_payer_names": primary_payer_names,
                    "search_term": search_term,
                    "external_id": external_id,
                    "diagnoses_updated_since": serialize_datetime(diagnoses_updated_since)
                    if diagnoses_updated_since is not None
                    else None,
                    "tag_ids": tag_ids,
                    "do_not_bill": do_not_bill,
                    "work_queue_id": work_queue_id,
                },
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
                timeout=60,
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(EncounterPage, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get(self, encounter_id: EncounterId) -> Encounter:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "GET",
                urllib.parse.urljoin(f"{self._environment.value}/", f"api/encounters/v4/{encounter_id}"),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
                timeout=60,
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Encounter, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def create(
        self,
        *,
        patient: PatientCreate,
        billing_provider: BillingProvider,
        rendering_provider: RenderingProvider,
        referring_provider: typing.Optional[ReferringProvider] = OMIT,
        service_facility: typing.Optional[EncounterServiceFacilityBase] = OMIT,
        subscriber_primary: typing.Optional[SubscriberCreate] = OMIT,
        subscriber_secondary: typing.Optional[SubscriberCreate] = OMIT,
        diagnoses: typing.List[DiagnosisCreate],
        clinical_notes: typing.Optional[typing.List[ClinicalNoteCategoryCreate]] = OMIT,
        billing_notes: typing.Optional[typing.List[BillingNoteBase]] = OMIT,
        place_of_service_code: FacilityTypeCode,
        patient_histories: typing.Optional[typing.List[PatientHistoryCategory]] = OMIT,
        service_lines: typing.Optional[typing.List[ServiceLineCreate]] = OMIT,
        external_id: typing.Optional[EncounterExternalId] = OMIT,
        date_of_service: Date,
        end_date_of_service: typing.Optional[Date] = OMIT,
        prior_authorization_number: typing.Optional[PriorAuthorizationNumber] = OMIT,
        patient_authorized_release: bool,
        benefits_assigned_to_provider: bool,
        provider_accepts_assignment: bool,
        appointment_type: typing.Optional[str] = OMIT,
        do_not_bill: typing.Optional[bool] = OMIT,
        existing_medications: typing.Optional[typing.List[Medication]] = OMIT,
        vitals: typing.Optional[Vitals] = OMIT,
        interventions: typing.Optional[typing.List[Intervention]] = OMIT,
        pay_to_address: typing.Optional[StreetAddressLongZip] = OMIT,
        synchronicity: typing.Optional[SynchronicityType] = OMIT,
    ) -> Encounter:
        _request: typing.Dict[str, typing.Any] = {
            "patient": patient,
            "billing_provider": billing_provider,
            "rendering_provider": rendering_provider,
            "diagnoses": diagnoses,
            "place_of_service_code": place_of_service_code,
            "date_of_service": date_of_service,
            "patient_authorized_release": patient_authorized_release,
            "benefits_assigned_to_provider": benefits_assigned_to_provider,
            "provider_accepts_assignment": provider_accepts_assignment,
        }
        if referring_provider is not OMIT:
            _request["referring_provider"] = referring_provider
        if service_facility is not OMIT:
            _request["service_facility"] = service_facility
        if subscriber_primary is not OMIT:
            _request["subscriber_primary"] = subscriber_primary
        if subscriber_secondary is not OMIT:
            _request["subscriber_secondary"] = subscriber_secondary
        if clinical_notes is not OMIT:
            _request["clinical_notes"] = clinical_notes
        if billing_notes is not OMIT:
            _request["billing_notes"] = billing_notes
        if patient_histories is not OMIT:
            _request["patient_histories"] = patient_histories
        if service_lines is not OMIT:
            _request["service_lines"] = service_lines
        if external_id is not OMIT:
            _request["external_id"] = external_id
        if end_date_of_service is not OMIT:
            _request["end_date_of_service"] = end_date_of_service
        if prior_authorization_number is not OMIT:
            _request["prior_authorization_number"] = prior_authorization_number
        if appointment_type is not OMIT:
            _request["appointment_type"] = appointment_type
        if do_not_bill is not OMIT:
            _request["do_not_bill"] = do_not_bill
        if existing_medications is not OMIT:
            _request["existing_medications"] = existing_medications
        if vitals is not OMIT:
            _request["vitals"] = vitals
        if interventions is not OMIT:
            _request["interventions"] = interventions
        if pay_to_address is not OMIT:
            _request["pay_to_address"] = pay_to_address
        if synchronicity is not OMIT:
            _request["synchronicity"] = synchronicity
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "POST",
                urllib.parse.urljoin(f"{self._environment.value}/", "api/encounters/v4"),
                json=jsonable_encoder(_request),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
                timeout=60,
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Encounter, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def update(
        self,
        encounter_id: EncounterId,
        *,
        prior_authorization_number: typing.Optional[PriorAuthorizationNumber] = OMIT,
        external_id: typing.Optional[EncounterExternalId] = OMIT,
        date_of_service: typing.Optional[Date] = OMIT,
        diagnosis_ids: typing.Optional[typing.List[DiagnosisId]] = OMIT,
        tag_ids: typing.Optional[typing.List[TagId]] = OMIT,
        clinical_notes: typing.Optional[typing.List[ClinicalNoteCategoryCreate]] = OMIT,
        pay_to_address: typing.Optional[StreetAddressLongZip] = OMIT,
    ) -> Encounter:
        _request: typing.Dict[str, typing.Any] = {}
        if prior_authorization_number is not OMIT:
            _request["prior_authorization_number"] = prior_authorization_number
        if external_id is not OMIT:
            _request["external_id"] = external_id
        if date_of_service is not OMIT:
            _request["date_of_service"] = date_of_service
        if diagnosis_ids is not OMIT:
            _request["diagnosis_ids"] = diagnosis_ids
        if tag_ids is not OMIT:
            _request["tag_ids"] = tag_ids
        if clinical_notes is not OMIT:
            _request["clinical_notes"] = clinical_notes
        if pay_to_address is not OMIT:
            _request["pay_to_address"] = pay_to_address
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "PATCH",
                urllib.parse.urljoin(f"{self._environment.value}/", f"api/encounters/v4/{encounter_id}"),
                json=jsonable_encoder(_request),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
                timeout=60,
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(Encounter, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def get_attachments(self, encounter_id: EncounterId) -> typing.List[EncounterAttachment]:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "GET",
                urllib.parse.urljoin(f"{self._environment.value}/", f"api/encounters/v4/{encounter_id}/attachments"),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
                timeout=60,
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(typing.List[EncounterAttachment], _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def generate_clinical_notes_pdf(self, encounter_id: EncounterId) -> GenerateClinicalNotesPdfResponse:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "POST",
                urllib.parse.urljoin(
                    f"{self._environment.value}/", f"api/encounters/v4/{encounter_id}/clinical-notes-pdf/generate"
                ),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
                timeout=60,
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(GenerateClinicalNotesPdfResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def recompute_network_status_results(self, encounter_id: EncounterId) -> NetworkStatusComputationResults:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "PUT",
                urllib.parse.urljoin(
                    f"{self._environment.value}/", f"api/encounters/v4/{encounter_id}/network-status-results"
                ),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
                timeout=60,
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(NetworkStatusComputationResults, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)

    async def mark_as_not_billable(self, encounter_id: EncounterId) -> MarkAsNotBillableResponse:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "PUT",
                urllib.parse.urljoin(
                    f"{self._environment.value}/", f"api/encounters/v4/{encounter_id}/mark-as-not-billable"
                ),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
                timeout=60,
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(MarkAsNotBillableResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

import httpx
import pydantic

from ...core.api_error import ApiError
from ...core.jsonable_encoder import jsonable_encoder
from ...core.remove_none_from_headers import remove_none_from_headers
from ...environment import CandidApiEnvironment
from ..commons.types.date import Date
from ..commons.types.insurance_type_code import InsuranceTypeCode
from ..commons.types.state import State
from .types.expected_network_status_response import ExpectedNetworkStatusResponse

# this is used as the default value for optional parameters
OMIT = typing.cast(typing.Any, ...)


class ExpectedNetworkStatusClient:
    def __init__(
        self, *, environment: CandidApiEnvironment = CandidApiEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    def compute(
        self,
        *,
        external_patient_id: typing.Optional[str] = OMIT,
        subscriber_payer_id: str,
        subscriber_payer_name: str,
        subscriber_insurance_type: typing.Optional[InsuranceTypeCode] = OMIT,
        subscriber_plan_name: typing.Optional[str] = OMIT,
        billing_provider_npi: str,
        billing_provider_tin: str,
        rendering_provider_npi: str,
        contracted_state: State,
        date_of_service: Date,
    ) -> ExpectedNetworkStatusResponse:
        _request: typing.Dict[str, typing.Any] = {
            "subscriber_payer_id": subscriber_payer_id,
            "subscriber_payer_name": subscriber_payer_name,
            "billing_provider_npi": billing_provider_npi,
            "billing_provider_tin": billing_provider_tin,
            "rendering_provider_npi": rendering_provider_npi,
            "contracted_state": contracted_state,
            "date_of_service": date_of_service,
        }
        if external_patient_id is not OMIT:
            _request["external_patient_id"] = external_patient_id
        if subscriber_insurance_type is not OMIT:
            _request["subscriber_insurance_type"] = subscriber_insurance_type
        if subscriber_plan_name is not OMIT:
            _request["subscriber_plan_name"] = subscriber_plan_name
        _response = httpx.request(
            "POST",
            urllib.parse.urljoin(f"{self._environment.value}/", "api/expected-network-status/v1"),
            json=jsonable_encoder(_request),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ExpectedNetworkStatusResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncExpectedNetworkStatusClient:
    def __init__(
        self, *, environment: CandidApiEnvironment = CandidApiEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    async def compute(
        self,
        *,
        external_patient_id: typing.Optional[str] = OMIT,
        subscriber_payer_id: str,
        subscriber_payer_name: str,
        subscriber_insurance_type: typing.Optional[InsuranceTypeCode] = OMIT,
        subscriber_plan_name: typing.Optional[str] = OMIT,
        billing_provider_npi: str,
        billing_provider_tin: str,
        rendering_provider_npi: str,
        contracted_state: State,
        date_of_service: Date,
    ) -> ExpectedNetworkStatusResponse:
        _request: typing.Dict[str, typing.Any] = {
            "subscriber_payer_id": subscriber_payer_id,
            "subscriber_payer_name": subscriber_payer_name,
            "billing_provider_npi": billing_provider_npi,
            "billing_provider_tin": billing_provider_tin,
            "rendering_provider_npi": rendering_provider_npi,
            "contracted_state": contracted_state,
            "date_of_service": date_of_service,
        }
        if external_patient_id is not OMIT:
            _request["external_patient_id"] = external_patient_id
        if subscriber_insurance_type is not OMIT:
            _request["subscriber_insurance_type"] = subscriber_insurance_type
        if subscriber_plan_name is not OMIT:
            _request["subscriber_plan_name"] = subscriber_plan_name
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "POST",
                urllib.parse.urljoin(f"{self._environment.value}/", "api/expected-network-status/v1"),
                json=jsonable_encoder(_request),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
                timeout=60,
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(ExpectedNetworkStatusResponse, _response_json)  # type: ignore
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .successful_generate_clinical_notes_pdf_response import SuccessfulGenerateClinicalNotesPdfResponse


class GenerateClinicalNotesPdfResponse_Success(SuccessfulGenerateClinicalNotesPdfResponse):
    result: typing_extensions.Literal["success"]

    class Config:
        frozen = True
        allow_population_by_field_name = True


GenerateClinicalNotesPdfResponse = typing.Union[GenerateClinicalNotesPdfResponse_Success]



# This file was auto-generated by Fern from our API Definition.

from .denial_reason_content import DenialReasonContent
from .drug_identification import DrugIdentification
from .measurement_unit_code import MeasurementUnitCode
from .service_id_qualifier import ServiceIdQualifier
from .service_line import ServiceLine
from .service_line_adjustment import ServiceLineAdjustment
from .service_line_base import ServiceLineBase
from .service_line_base_with_optionals import ServiceLineBaseWithOptionals
from .service_line_create import ServiceLineCreate
from .service_line_denial_reason import ServiceLineDenialReason
from .service_line_era_data import ServiceLineEraData

__all__ = [
    "DenialReasonContent",
    "DrugIdentification",
    "MeasurementUnitCode",
    "ServiceIdQualifier",
    "ServiceLine",
    "ServiceLineAdjustment",
    "ServiceLineBase",
    "ServiceLineBaseWithOptionals",
    "ServiceLineCreate",
    "ServiceLineDenialReason",
    "ServiceLineEraData",
]



# This file was auto-generated by Fern from our API Definition.

from .work_queue import WorkQueue
from .work_queue_category import WorkQueueCategory
from .work_queue_category_type import WorkQueueCategoryType

__all__ = ["WorkQueue", "WorkQueueCategory", "WorkQueueCategoryType"]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...commons.types.date import Date
from ...commons.types.patient_relationship_to_insured_code_all import PatientRelationshipToInsuredCodeAll
from ...commons.types.street_address_short_zip import StreetAddressShortZip
from .individual_base import IndividualBase


class SubscriberBase(IndividualBase):
    patient_relationship_to_subscriber_code: PatientRelationshipToInsuredCodeAll
    date_of_birth: typing.Optional[Date]
    address: typing.Optional[StreetAddressShortZip]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class EncounterAttachmentType(str, enum.Enum):
    DOCUMENTATION = "DOCUMENTATION"

    def visit(self, documentation: typing.Callable[[], T_Result]) -> T_Result:
        if self is EncounterAttachmentType.DOCUMENTATION:
            return documentation()



# This file was auto-generated by Fern from our API Definition.

from .credentialed_encounter_status_result import CredentialedEncounterStatusResult
from .credentialing_span_dates import (
    CredentialingSpanDates,
    CredentialingSpanDates_NonRequiredDates,
    CredentialingSpanDates_RequiredDates,
)
from .credentialing_span_status import CredentialingSpanStatus
from .encounter_credentialing_status_result import (
    EncounterCredentialingStatusResult,
    EncounterCredentialingStatusResult_Credentialed,
    EncounterCredentialingStatusResult_NotCredentialed,
)
from .non_required_credentialing_dates import NonRequiredCredentialingDates
from .provider_credentialing_span import ProviderCredentialingSpan
from .provider_credentialing_span_base import ProviderCredentialingSpanBase
from .provider_credentialing_span_id import ProviderCredentialingSpanId
from .required_credentialing_dates import RequiredCredentialingDates

__all__ = [
    "CredentialedEncounterStatusResult",
    "CredentialingSpanDates",
    "CredentialingSpanDates_NonRequiredDates",
    "CredentialingSpanDates_RequiredDates",
    "CredentialingSpanStatus",
    "EncounterCredentialingStatusResult",
    "EncounterCredentialingStatusResult_Credentialed",
    "EncounterCredentialingStatusResult_NotCredentialed",
    "NonRequiredCredentialingDates",
    "ProviderCredentialingSpan",
    "ProviderCredentialingSpanBase",
    "ProviderCredentialingSpanId",
    "RequiredCredentialingDates",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from ...commons.types.date import Date
from ...commons.types.regions import Regions
from .authorized_signatory import AuthorizedSignatory
from .contract_status import ContractStatus


class ContractBase(pydantic.BaseModel):
    effective_date: Date = pydantic.Field(description=("The starting day upon which the contract is effective\n"))
    expiration_date: typing.Optional[Date] = pydantic.Field(
        description=("An optional end day upon which the contract expires\n")
    )
    regions: Regions = pydantic.Field(
        description=(
            "The state(s) to which the contract's coverage extends.\n"
            'It may also be set to "national" for the entirety of the US.\n'
        )
    )
    contract_status: typing.Optional[ContractStatus]
    authorized_signatory: typing.Optional[AuthorizedSignatory]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .auth_0_id import Auth0Id
from .claim_id import ClaimId
from .content_download_url import ContentDownloadUrl
from .date import Date
from .encounter_external_id import EncounterExternalId
from .encounter_id import EncounterId
from .facility_type_code import FacilityTypeCode
from .insurance_type_code import InsuranceTypeCode
from .link_url import LinkUrl
from .npi import Npi
from .organization_id import OrganizationId
from .page_token import PageToken
from .patient_external_id import PatientExternalId
from .patient_relationship_to_insured_code_all import PatientRelationshipToInsuredCodeAll
from .procedure_modifier import ProcedureModifier
from .region_national import RegionNational
from .region_states import RegionStates
from .regions import Regions, Regions_National, Regions_States
from .resource_page import ResourcePage
from .service_line_id import ServiceLineId
from .service_line_units import ServiceLineUnits
from .source_of_payment_code import SourceOfPaymentCode
from .state import State
from .street_address_base import StreetAddressBase
from .street_address_long_zip import StreetAddressLongZip
from .street_address_short_zip import StreetAddressShortZip
from .work_queue_id import WorkQueueId

__all__ = [
    "Auth0Id",
    "ClaimId",
    "ContentDownloadUrl",
    "Date",
    "EncounterExternalId",
    "EncounterId",
    "FacilityTypeCode",
    "InsuranceTypeCode",
    "LinkUrl",
    "Npi",
    "OrganizationId",
    "PageToken",
    "PatientExternalId",
    "PatientRelationshipToInsuredCodeAll",
    "ProcedureModifier",
    "RegionNational",
    "RegionStates",
    "Regions",
    "Regions_National",
    "Regions_States",
    "ResourcePage",
    "ServiceLineId",
    "ServiceLineUnits",
    "SourceOfPaymentCode",
    "State",
    "StreetAddressBase",
    "StreetAddressLongZip",
    "StreetAddressShortZip",
    "WorkQueueId",
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...commons.types.claim_id import ClaimId
from ...commons.types.service_line_id import ServiceLineId
from ...commons.types.service_line_units import ServiceLineUnits
from .service_line_base_with_optionals import ServiceLineBaseWithOptionals


class ServiceLine(ServiceLineBaseWithOptionals):
    service_line_id: ServiceLineId
    procedure_code: str
    quantity: str
    units: ServiceLineUnits
    claim_id: ClaimId

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from ...commons.types.facility_type_code import FacilityTypeCode
from ...diagnoses.types.diagnosis_id import DiagnosisId
from ...invoices.types.invoice import Invoice
from .service_line_adjustment import ServiceLineAdjustment
from .service_line_base import ServiceLineBase
from .service_line_denial_reason import ServiceLineDenialReason
from .service_line_era_data import ServiceLineEraData


class ServiceLineBaseWithOptionals(ServiceLineBase):
    charge_amount_cents: typing.Optional[int]
    allowed_amount_cents: typing.Optional[int]
    paid_amount_cents: typing.Optional[int]
    patient_responsibility_cents: typing.Optional[int]
    diagnosis_id_zero: typing.Optional[DiagnosisId]
    diagnosis_id_one: typing.Optional[DiagnosisId]
    diagnosis_id_two: typing.Optional[DiagnosisId]
    diagnosis_id_three: typing.Optional[DiagnosisId]
    service_line_era_data: typing.Optional[ServiceLineEraData]
    service_line_manual_adjustments: typing.Optional[typing.List[ServiceLineAdjustment]]
    related_invoices: typing.Optional[typing.List[Invoice]]
    denial_reason: typing.Optional[ServiceLineDenialReason]
    place_of_service_code: typing.Optional[FacilityTypeCode]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from ...commons.types.service_line_units import ServiceLineUnits
from .drug_identification import DrugIdentification
from .service_line_base import ServiceLineBase


class ServiceLineCreate(ServiceLineBase):
    procedure_code: str
    quantity: str
    units: ServiceLineUnits
    charge_amount_cents: typing.Optional[int] = pydantic.Field(
        description=(
            "The total amount charged for this service line taking quantity into account. For example, if a single unit\n"
            "costs 100 cents and 2 units were rendered, the charge_amount_cents should be 200. Should be greater than or\n"
            "equal to 0.\n"
        )
    )
    diagnosis_pointers: typing.List[int] = pydantic.Field(
        description=("Indices (zero-indexed) of all the diagnoses this service line references\n")
    )
    drug_identification: typing.Optional[DrugIdentification]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions

from .in_network_contracting_status_result import InNetworkContractingStatusResult
from .out_of_network_contracting_status_result import OutOfNetworkContractingStatusResult


class AttributableContractingStatusResult_OutOfNetwork(OutOfNetworkContractingStatusResult):
    contracting_status: typing_extensions.Literal["out_of_network"]

    class Config:
        frozen = True
        allow_population_by_field_name = True


class AttributableContractingStatusResult_InNetwork(InNetworkContractingStatusResult):
    contracting_status: typing_extensions.Literal["in_network"]

    class Config:
        frozen = True
        allow_population_by_field_name = True


class AttributableContractingStatusResult_Unknown(pydantic.BaseModel):
    contracting_status: typing_extensions.Literal["unknown"]

    class Config:
        frozen = True


AttributableContractingStatusResult = typing.Union[
    AttributableContractingStatusResult_OutOfNetwork,
    AttributableContractingStatusResult_InNetwork,
    AttributableContractingStatusResult_Unknown,
]



# This file was auto-generated by Fern from our API Definition.

import typing
import urllib.parse
from json.decoder import JSONDecodeError

import httpx
import pydantic

from .....core.api_error import ApiError
from .....core.jsonable_encoder import jsonable_encoder
from .....core.remove_none_from_headers import remove_none_from_headers
from .....environment import CandidApiEnvironment
from .errors.too_many_requests_error import TooManyRequestsError
from .types.auth_get_token_request import AuthGetTokenRequest
from .types.auth_get_token_response import AuthGetTokenResponse
from .types.too_many_requests_error_type import TooManyRequestsErrorType


class V2Client:
    def __init__(
        self, *, environment: CandidApiEnvironment = CandidApiEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    def get_token(self, *, request: AuthGetTokenRequest) -> AuthGetTokenResponse:
        _response = httpx.request(
            "POST",
            urllib.parse.urljoin(f"{self._environment.value}/", "api/auth/v2/token"),
            json=jsonable_encoder(request),
            headers=remove_none_from_headers(
                {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
            ),
            timeout=60,
        )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(AuthGetTokenResponse, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "TooManyRequestsError":
                raise TooManyRequestsError(
                    pydantic.parse_obj_as(TooManyRequestsErrorType, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)


class AsyncV2Client:
    def __init__(
        self, *, environment: CandidApiEnvironment = CandidApiEnvironment.PRODUCTION, token: typing.Optional[str] = None
    ):
        self._environment = environment
        self._token = token

    async def get_token(self, *, request: AuthGetTokenRequest) -> AuthGetTokenResponse:
        async with httpx.AsyncClient() as _client:
            _response = await _client.request(
                "POST",
                urllib.parse.urljoin(f"{self._environment.value}/", "api/auth/v2/token"),
                json=jsonable_encoder(request),
                headers=remove_none_from_headers(
                    {"Authorization": f"Bearer {self._token}" if self._token is not None else None}
                ),
                timeout=60,
            )
        try:
            _response_json = _response.json()
        except JSONDecodeError:
            raise ApiError(status_code=_response.status_code, body=_response.text)
        if 200 <= _response.status_code < 300:
            return pydantic.parse_obj_as(AuthGetTokenResponse, _response_json)  # type: ignore
        if "errorName" in _response_json:
            if _response_json["errorName"] == "TooManyRequestsError":
                raise TooManyRequestsError(
                    pydantic.parse_obj_as(TooManyRequestsErrorType, _response_json["content"])  # type: ignore
                )
        raise ApiError(status_code=_response.status_code, body=_response_json)



# This file was auto-generated by Fern from our API Definition.

from .attachment_id import AttachmentId
from .attributable_contracting_status_result import (
    AttributableContractingStatusResult,
    AttributableContractingStatusResult_InNetwork,
    AttributableContractingStatusResult_OutOfNetwork,
    AttributableContractingStatusResult_Unknown,
)
from .base_attachment import BaseAttachment
from .clinical_note import ClinicalNote
from .clinical_note_category import ClinicalNoteCategory
from .clinical_note_category_create import ClinicalNoteCategoryCreate
from .coding_attribution_type import CodingAttributionType
from .contracting_out_of_of_network_reason import ContractingOutOfOfNetworkReason
from .encounter import Encounter
from .encounter_attachment import EncounterAttachment
from .encounter_attachment_type import EncounterAttachmentType
from .encounter_base import EncounterBase
from .encounter_page import EncounterPage
from .encounter_sort_options import EncounterSortOptions
from .generate_clinical_notes_pdf_response import (
    GenerateClinicalNotesPdfResponse,
    GenerateClinicalNotesPdfResponse_Success,
)
from .in_network_contracting_status_result import InNetworkContractingStatusResult
from .intake_follow_up import IntakeFollowUp
from .intake_follow_up_id import IntakeFollowUpId
from .intake_question import IntakeQuestion
from .intake_question_id import IntakeQuestionId
from .intake_response_and_follow_ups import IntakeResponseAndFollowUps
from .intervention import Intervention
from .intervention_category import InterventionCategory
from .lab import Lab
from .lab_code_type import LabCodeType
from .mark_as_not_billable_response import MarkAsNotBillableResponse
from .medication import Medication
from .network_status import NetworkStatus
from .network_status_computation_results import NetworkStatusComputationResults
from .note_category import NoteCategory
from .out_of_network_contracting_status_result import OutOfNetworkContractingStatusResult
from .patient_history_category import PatientHistoryCategory
from .patient_history_category_enum import PatientHistoryCategoryEnum
from .prior_authorization_number import PriorAuthorizationNumber
from .rx_cui import RxCui
from .successful_generate_clinical_notes_pdf_response import SuccessfulGenerateClinicalNotesPdfResponse
from .synchronicity_type import SynchronicityType
from .vitals import Vitals

__all__ = [
    "AttachmentId",
    "AttributableContractingStatusResult",
    "AttributableContractingStatusResult_InNetwork",
    "AttributableContractingStatusResult_OutOfNetwork",
    "AttributableContractingStatusResult_Unknown",
    "BaseAttachment",
    "ClinicalNote",
    "ClinicalNoteCategory",
    "ClinicalNoteCategoryCreate",
    "CodingAttributionType",
    "ContractingOutOfOfNetworkReason",
    "Encounter",
    "EncounterAttachment",
    "EncounterAttachmentType",
    "EncounterBase",
    "EncounterPage",
    "EncounterSortOptions",
    "GenerateClinicalNotesPdfResponse",
    "GenerateClinicalNotesPdfResponse_Success",
    "InNetworkContractingStatusResult",
    "IntakeFollowUp",
    "IntakeFollowUpId",
    "IntakeQuestion",
    "IntakeQuestionId",
    "IntakeResponseAndFollowUps",
    "Intervention",
    "InterventionCategory",
    "Lab",
    "LabCodeType",
    "MarkAsNotBillableResponse",
    "Medication",
    "NetworkStatus",
    "NetworkStatusComputationResults",
    "NoteCategory",
    "OutOfNetworkContractingStatusResult",
    "PatientHistoryCategory",
    "PatientHistoryCategoryEnum",
    "PriorAuthorizationNumber",
    "RxCui",
    "SuccessfulGenerateClinicalNotesPdfResponse",
    "SynchronicityType",
    "Vitals",
]



# This file was auto-generated by Fern from our API Definition.

from .authorized_signatory import AuthorizedSignatory
from .contract import Contract
from .contract_base import ContractBase
from .contract_id import ContractId
from .contract_status import ContractStatus

__all__ = ["AuthorizedSignatory", "Contract", "ContractBase", "ContractId", "ContractStatus"]



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import typing_extensions

from .non_required_credentialing_dates import NonRequiredCredentialingDates
from .required_credentialing_dates import RequiredCredentialingDates


class CredentialingSpanDates_RequiredDates(RequiredCredentialingDates):
    type: typing_extensions.Literal["required_dates"]

    class Config:
        frozen = True
        allow_population_by_field_name = True


class CredentialingSpanDates_NonRequiredDates(NonRequiredCredentialingDates):
    type: typing_extensions.Literal["non_required_dates"]

    class Config:
        frozen = True
        allow_population_by_field_name = True


CredentialingSpanDates = typing.Union[CredentialingSpanDates_RequiredDates, CredentialingSpanDates_NonRequiredDates]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ......core.datetime_utils import serialize_datetime
from .....commons.types.date import Date
from .....commons.types.encounter_external_id import EncounterExternalId
from .....commons.types.street_address_long_zip import StreetAddressLongZip
from .intervention import Intervention
from .medication import Medication
from .prior_authorization_number import PriorAuthorizationNumber
from .synchronicity_type import SynchronicityType
from .vitals import Vitals


class EncounterBase(pydantic.BaseModel):
    external_id: typing.Optional[EncounterExternalId] = pydantic.Field(
        description=(
            "A client-specified unique ID to associate with this encounter;\n"
            "for example, your internal encounter ID or a Dr. Chrono encounter ID.\n"
            "This field should not contain PHI.\n"
        )
    )
    date_of_service: Date = pydantic.Field(
        description=(
            "Date formatted as YYYY-MM-DD; eg: 2019-08-24.\n"
            "This date must be the local date in the timezone where the service occurred.\n"
            "Box 24a on the CMS-1500 claim form.\n"
            "If service occurred over a range of dates, this should be the start date.\n"
        )
    )
    end_date_of_service: typing.Optional[Date] = pydantic.Field(
        description=(
            "Date formatted as YYYY-MM-DD; eg: 2019-08-25.\n"
            "This date must be the local date in the timezone where the service occurred.\n"
            "If omitted, the Encounter is assumed to be for a single day.\n"
            "Must not be temporally before the date_of_service field.\n"
        )
    )
    prior_authorization_number: typing.Optional[PriorAuthorizationNumber] = pydantic.Field(
        description=("Box 23 on the CMS-1500 claim form.\n")
    )
    patient_authorized_release: bool = pydantic.Field(
        description=(
            "Whether this patient has authorized the release of medical information\n"
            "for billing purpose.\n"
            "Box 12 on the CMS-1500 claim form.\n"
        )
    )
    benefits_assigned_to_provider: bool = pydantic.Field(
        description=(
            "Whether this patient has authorized insurance payments to be made to you,\n"
            "not them. If false, patient may receive reimbursement.\n"
            "Box 13 on the CMS-1500 claim form.\n"
        )
    )
    provider_accepts_assignment: bool = pydantic.Field(
        description=(
            "Whether you have accepted the patient's authorization for insurance payments\n"
            "to be made to you, not them.\n"
            "Box 27 on the CMS-1500 claim form.\n"
        )
    )
    appointment_type: typing.Optional[str] = pydantic.Field(
        description=('Human-readable description of the appointment time (ex: "Acupuncture - Headaches")\n')
    )
    do_not_bill: typing.Optional[bool] = pydantic.Field(
        description=(
            "Should be set to true if Candid should not create or submit a claim but you'd\n"
            "like us to track this encounter anyway (ex: patient is paying cash)\n"
        )
    )
    existing_medications: typing.Optional[typing.List[Medication]]
    vitals: typing.Optional[Vitals]
    interventions: typing.Optional[typing.List[Intervention]]
    pay_to_address: typing.Optional[StreetAddressLongZip]
    synchronicity: typing.Optional[SynchronicityType] = pydantic.Field(
        description=(
            "Whether or not this was a synchronous or asynchronous encounter.\n"
            "Asynchronous encounters occur when providers and patients communicate online using\n"
            "forms, instant messaging, or other pre-recorded digital mediums.\n"
            "Synchronous encounters occur in live, real-time settings where the patient interacts\n"
            "directly with the provider, such as over video or a phone call.\n"
        )
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum


class CandidApiEnvironment(enum.Enum):
    PRODUCTION = "https://api.joincandidhealth.com"
    STAGING = "https://api-staging.joincandidhealth.com"



# This file was auto-generated by Fern from our API Definition.

from .types import (
    AttachmentId,
    AttributableContractingStatusResult,
    AttributableContractingStatusResult_InNetwork,
    AttributableContractingStatusResult_OutOfNetwork,
    AttributableContractingStatusResult_Unknown,
    BaseAttachment,
    ClinicalNote,
    ClinicalNoteCategory,
    ClinicalNoteCategoryCreate,
    CodingAttributionType,
    ContractingOutOfOfNetworkReason,
    Encounter,
    EncounterAttachment,
    EncounterAttachmentType,
    EncounterBase,
    EncounterPage,
    EncounterSortOptions,
    GenerateClinicalNotesPdfResponse,
    GenerateClinicalNotesPdfResponse_Success,
    InNetworkContractingStatusResult,
    IntakeFollowUp,
    IntakeFollowUpId,
    IntakeQuestion,
    IntakeQuestionId,
    IntakeResponseAndFollowUps,
    Intervention,
    InterventionCategory,
    Lab,
    LabCodeType,
    MarkAsNotBillableResponse,
    Medication,
    NetworkStatus,
    NetworkStatusComputationResults,
    NoteCategory,
    OutOfNetworkContractingStatusResult,
    PatientHistoryCategory,
    PatientHistoryCategoryEnum,
    PriorAuthorizationNumber,
    RxCui,
    SuccessfulGenerateClinicalNotesPdfResponse,
    SynchronicityType,
    Vitals,
)

__all__ = [
    "AttachmentId",
    "AttributableContractingStatusResult",
    "AttributableContractingStatusResult_InNetwork",
    "AttributableContractingStatusResult_OutOfNetwork",
    "AttributableContractingStatusResult_Unknown",
    "BaseAttachment",
    "ClinicalNote",
    "ClinicalNoteCategory",
    "ClinicalNoteCategoryCreate",
    "CodingAttributionType",
    "ContractingOutOfOfNetworkReason",
    "Encounter",
    "EncounterAttachment",
    "EncounterAttachmentType",
    "EncounterBase",
    "EncounterPage",
    "EncounterSortOptions",
    "GenerateClinicalNotesPdfResponse",
    "GenerateClinicalNotesPdfResponse_Success",
    "InNetworkContractingStatusResult",
    "IntakeFollowUp",
    "IntakeFollowUpId",
    "IntakeQuestion",
    "IntakeQuestionId",
    "IntakeResponseAndFollowUps",
    "Intervention",
    "InterventionCategory",
    "Lab",
    "LabCodeType",
    "MarkAsNotBillableResponse",
    "Medication",
    "NetworkStatus",
    "NetworkStatusComputationResults",
    "NoteCategory",
    "OutOfNetworkContractingStatusResult",
    "PatientHistoryCategory",
    "PatientHistoryCategoryEnum",
    "PriorAuthorizationNumber",
    "RxCui",
    "SuccessfulGenerateClinicalNotesPdfResponse",
    "SynchronicityType",
    "Vitals",
]



# This file was auto-generated by Fern from our API Definition.

from .types import AuthorizedSignatory, Contract, ContractBase, ContractId, ContractStatus

__all__ = ["AuthorizedSignatory", "Contract", "ContractBase", "ContractId", "ContractStatus"]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ......core.datetime_utils import serialize_datetime
from .....billing_notes.types.billing_note import BillingNote
from .....claims.types.claim import Claim
from .....commons.types.encounter_id import EncounterId
from .....commons.types.facility_type_code import FacilityTypeCode
from .....commons.types.link_url import LinkUrl
from .....diagnoses.types.diagnosis import Diagnosis
from .....encounter_providers.resources.v_2.types.encounter_provider import EncounterProvider
from .....individual.types.patient import Patient
from .....individual.types.subscriber import Subscriber
from .....patient_payments.resources.v_2.types.patient_payment import PatientPayment
from .....service_facility.types.encounter_service_facility import EncounterServiceFacility
from .....tags.types.tag import Tag
from .....work_queues.resources.v_1.types.work_queue import WorkQueue
from .clinical_note_category import ClinicalNoteCategory
from .coding_attribution_type import CodingAttributionType
from .encounter_base import EncounterBase
from .patient_history_category import PatientHistoryCategory


class Encounter(EncounterBase):
    encounter_id: EncounterId
    claims: typing.List[Claim]
    patient: Patient
    billing_provider: EncounterProvider
    rendering_provider: EncounterProvider
    referring_provider: typing.Optional[EncounterProvider]
    service_facility: EncounterServiceFacility
    subscriber_primary: typing.Optional[Subscriber]
    subscriber_secondary: typing.Optional[Subscriber]
    url: LinkUrl = pydantic.Field(description=("URL that links directly to the claim created in Candid\n"))
    diagnoses: typing.List[Diagnosis]
    clinical_notes: typing.List[ClinicalNoteCategory]
    billing_notes: typing.Optional[typing.List[BillingNote]] = pydantic.Field(
        description=(
            "Spot to store misc, human-readable, notes about this encounter to be\n" "used in the billing process.\n"
        )
    )
    place_of_service_code: typing.Optional[FacilityTypeCode]
    place_of_service_code_as_submitted: typing.Optional[FacilityTypeCode]
    patient_histories: typing.List[PatientHistoryCategory]
    patient_payments: typing.List[PatientPayment]
    tags: typing.List[Tag]
    coding_attribution: typing.Optional[CodingAttributionType]
    work_queue: typing.Optional[WorkQueue]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ....core.datetime_utils import serialize_datetime
from ...commons.types.date import Date
from ...commons.types.street_address_short_zip import StreetAddressShortZip
from .individual_base import IndividualBase


class PatientCreate(IndividualBase):
    external_id: str = pydantic.Field(
        description=(
            "Another ID you want to associate with this patient.\n"
            "For example, your internal patient ID or a Dr. Chrono patient ID. Box 1a on the CMS-1500 claim form\n"
        )
    )
    date_of_birth: Date = pydantic.Field(
        description=(
            "Box 3 on the CMS-1500 claim form. The date format should be in ISO 8601 date; formatted YYYY-MM-DD (i.e. 2012-02-01)\n"
        )
    )
    address: StreetAddressShortZip = pydantic.Field(description=("Box 5 on the CMS-1500 claim form.\n"))

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class ContractingOutOfOfNetworkReason(str, enum.Enum):
    NO_EFFECTIVE_CONTRACT = "no_effective_contract"
    MISSING_CONTRACT_LINKAGE = "missing_contract_linkage"

    def visit(
        self,
        no_effective_contract: typing.Callable[[], T_Result],
        missing_contract_linkage: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is ContractingOutOfOfNetworkReason.NO_EFFECTIVE_CONTRACT:
            return no_effective_contract()
        if self is ContractingOutOfOfNetworkReason.MISSING_CONTRACT_LINKAGE:
            return missing_contract_linkage()



# This file was auto-generated by Fern from our API Definition.

from __future__ import annotations

import typing

import pydantic
import typing_extensions

from .credentialed_encounter_status_result import CredentialedEncounterStatusResult


class EncounterCredentialingStatusResult_Credentialed(CredentialedEncounterStatusResult):
    type: typing_extensions.Literal["credentialed"]

    class Config:
        frozen = True
        allow_population_by_field_name = True


class EncounterCredentialingStatusResult_NotCredentialed(pydantic.BaseModel):
    type: typing_extensions.Literal["not_credentialed"]

    class Config:
        frozen = True


EncounterCredentialingStatusResult = typing.Union[
    EncounterCredentialingStatusResult_Credentialed, EncounterCredentialingStatusResult_NotCredentialed
]



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

from ....core.datetime_utils import serialize_datetime
from .insurance_card_base import InsuranceCardBase
from .insurance_card_id import InsuranceCardId


class InsuranceCard(InsuranceCardBase):
    insurance_card_id: InsuranceCardId
    member_id: str
    payer_name: str
    payer_id: str
    rx_bin: typing.Optional[str]
    rx_pcn: typing.Optional[str]
    image_url_front: typing.Optional[str]
    image_url_back: typing.Optional[str]

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        allow_population_by_field_name = True
        json_encoders = {dt.datetime: serialize_datetime}



# This file was auto-generated by Fern from our API Definition.

from .types import (
    AddressType,
    EmploymentStatus,
    LicenseType,
    OrganizationProvider,
    OrganizationProviderAddress,
    OrganizationProviderBase,
    OrganizationProviderId,
    ProviderType,
)

__all__ = [
    "AddressType",
    "EmploymentStatus",
    "LicenseType",
    "OrganizationProvider",
    "OrganizationProviderAddress",
    "OrganizationProviderBase",
    "OrganizationProviderId",
    "ProviderType",
]



# This file was auto-generated by Fern from our API Definition.

from .types import (
    Auth0Id,
    ClaimId,
    ContentDownloadUrl,
    Date,
    EncounterExternalId,
    EncounterId,
    FacilityTypeCode,
    InsuranceTypeCode,
    LinkUrl,
    Npi,
    OrganizationId,
    PageToken,
    PatientExternalId,
    PatientRelationshipToInsuredCodeAll,
    ProcedureModifier,
    RegionNational,
    Regions,
    Regions_National,
    Regions_States,
    RegionStates,
    ResourcePage,
    ServiceLineId,
    ServiceLineUnits,
    SourceOfPaymentCode,
    State,
    StreetAddressBase,
    StreetAddressLongZip,
    StreetAddressShortZip,
    WorkQueueId,
)

__all__ = [
    "Auth0Id",
    "ClaimId",
    "ContentDownloadUrl",
    "Date",
    "EncounterExternalId",
    "EncounterId",
    "FacilityTypeCode",
    "InsuranceTypeCode",
    "LinkUrl",
    "Npi",
    "OrganizationId",
    "PageToken",
    "PatientExternalId",
    "PatientRelationshipToInsuredCodeAll",
    "ProcedureModifier",
    "RegionNational",
    "RegionStates",
    "Regions",
    "Regions_National",
    "Regions_States",
    "ResourcePage",
    "ServiceLineId",
    "ServiceLineUnits",
    "SourceOfPaymentCode",
    "State",
    "StreetAddressBase",
    "StreetAddressLongZip",
    "StreetAddressShortZip",
    "WorkQueueId",
]



# This file was auto-generated by Fern from our API Definition.

import enum
import typing

T_Result = typing.TypeVar("T_Result")


class NetworkStatus(str, enum.Enum):
    IN_NETWORK = "in_network"
    OUT_OF_NETWORK = "out_of_network"
    UNKNOWN = "unknown"
    NOT_ADJUDICATED = "not_adjudicated"

    def visit(
        self,
        in_network: typing.Callable[[], T_Result],
        out_of_network: typing.Callable[[], T_Result],
        unknown: typing.Callable[[], T_Result],
        not_adjudicated: typing.Callable[[], T_Result],
    ) -> T_Result:
        if self is NetworkStatus.IN_NETWORK:
            return in_network()
        if self is NetworkStatus.OUT_OF_NETWORK:
            return out_of_network()
        if self is NetworkStatus.UNKNOWN:
            return unknown()
        if self is NetworkStatus.NOT_ADJUDICATED:
            return not_adjudicated()



# This file was auto-generated by Fern from our API Definition.

import datetime as dt
import typing

import pydantic

from ......core.datetime_utils import serialize_datetime
from .license_type import LicenseType
from .organization_provider_address import OrganizationProviderAddress
from .provider_type import ProviderType


class OrganizationProviderBase(pydantic.BaseModel):
    npi: str = pydantic.Field(
        description=("The NPI of the provider. This must be all digits [0-9] and exactly 10 characters long.\n")
    )
    is_rendering: bool = pydantic.Field(description=("Whether the provider can be used to render services.\n"))
    is_billing: bool = pydantic.Field(description=("Whether the provider can be used to bill services.\n"))
    first_name: typing.Optional[str] = pydantic.Field(
        description=("The first name of the provider, if the provider is an individual.\n")
    )
    last_name: typing.Optional[str] = pydantic.Field(
        description=("The last name of the provider, if the provider is an individual.\n")
    )
    organization_name: typing.Optional[str] = pydantic.Field(
        description=("The name of the provider, if the provider is an organization.\n")
    )
    provider_type: ProviderType = pydantic.Field(
        description=("Whether the provider is an individual (NPPES Type 1) or organization (NPPES Type 2) provider.\n")
    )
    tax_id: typing.Optional[str] = pydantic.Field(
        description=(
            "If the provider has a contract with insurance, this must be the same tax ID given to the payer on an IRS W-9 form completed during contracting.\n"
        )
    )
    taxonomy_code: typing.Optional[str] = pydantic.Field(
        description=("A code designating classification and specialization.\n")
    )
    license_type: LicenseType = pydantic.Field(description=("The type of license that the provider holds.\n"))
    ptan: typing.Optional[str] = pydantic.Field(
        description=("The Provider Transaction Access Number for the provider.\n")
    )
    medicaid_provider_id: typing.Optional[str] = pydantic.Field(
        description=("The medicaid provider ID for the provider.\n")
    )
    addresses: typing.Optional[typing.List[OrganizationProviderAddress]] = pydantic.Field(
        description=("The addresses associated with this provider.\n")
    )

    def json(self, **kwargs: typing.Any) -> str:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().json(**kwargs_with_defaults)

    def dict(self, **kwargs: typing.Any) -> typing.Dict[str, typing.Any]:
        kwargs_with_defaults: typing.Any = {"by_alias": True, "exclude_unset": True, **kwargs}
        return super().dict(**kwargs_with_defaults)

    class Config:
        frozen = True
        json_encoders = {dt.datetime: serialize_datetime}
